package dm_msg;
require Exporter;
@ISA    = qw(Exporter);
@EXPORT = qw(send_msgs);

#    Devmon: An SNMP data collector & page generator for the BigBrother &
#    Hobbit network monitoring systems
#    Copyright (C) 2005-2006  Eric Schwimmer
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.  Please see the file named
#    'COPYING' that was included with the distrubition for more details.


 # Modules
  use strict;
  use Socket;
  use dm_config;

 # Our global variable hash
  use vars qw(%g);
  *g = \%dm_config::g;

 # Send our test results to the BB/Hobbit server
  sub send_msgs {
    $g{'msgxfrtime'} = time;

    do_log('DEBUG MSG: running send_msgs()',1) if $g{'debug'};
    do_log('Sending messages to display server',1);

   # Determine the address we are connecting to
    my $host = $g{'dispserv'};
    my $addr = inet_aton($host) 
      or do_log("Can't resolve display server $host ($!)",0) and return;
    my $p_addr = sockaddr_in($g{'dispport'}, $addr);

   # Print messages to stdout if requested
    if($g{'print_msg'} and defined $g{'test_results'}) {
      print join "\n", @{$g{'test_results'}};
    }
    
   # Dont actually send messages if we are printing them or in debug mode
    if($g{'print_msg'} or $g{'debug'}) {
      $g{'msgxfrtime'} = time - $g{'msgxfrtime'};
      print dm_stat_msg();
      return;
    }

   # Run until we are out of messages to send
    SOCKLOOP: while(@{$g{'test_results'}}) {

     # Rest for a moment so we dont overwhelm our display server
      select undef, undef, undef, $g{'msgsleep'} / 1000;

     # Open our socket to the host
      socket(SOCK, PF_INET, SOCK_STREAM, getprotobyname('tcp'))
        or do_log("Failed to create socket ($!)",0) and 
        $g{'msgxfrtime'} = time - $g{'msgxfrtime'} and return; 
      if (!connect(SOCK, $p_addr)) {
        do_log("Can't connect to display server $host ($!)",0);
        $g{'msgxfrtime'} = time - $g{'msgxfrtime'};
        close SOCK;
        return;
      }

     # Tell the display server that we are sending a combo msg
      print SOCK "combo\n";

     # Now print to this socket until we hit the max msg size
      my $msg_size = 0;
      MSGLOOP: while($msg_size < $g{'msgsize'} 
                     and @{$g{'test_results'}}) { 
        my $msg = shift @{$g{'test_results'}};
       # Make sure this is a valid message
        if(!defined $msg or $msg eq '') {
          do_log("Error: dm_msg trying to send a blank message!",0);
          next MSGLOOP;
        }
        
       # Make sure the message itself isnt too big
        if(length $msg > $g{'msgsize'}) {
        
         # Nuts, this is a huge message, bigger than our msg size. Well want
         # to send it by itself to minimize how much it gets truncated
          if($msg_size == 0) {
           # Okay, we are clear, send the message 
            print SOCK "$msg\n";
          }

         # Not an empty combo msg, wait till our new socket is open
          else { 
            unshift @{$g{'test_results'}}, $msg;
          }

         # Either way, open a new socket
          close SOCK;
          next SOCKLOOP;
        }
    
       # Now make sure that this msg wont cause our current combo msg to
       # exceed the msgsize limit
        elsif($msg_size + length $msg > $g{'msgsize'}) {
         
         # Whoops, looks like it does;  wait for the next socket
          unshift @{$g{'test_results'}}, $msg; 
          close SOCK;
          next SOCKLOOP;
        }

       # Looks good, print the msg           
        else {
          print SOCK "$msg\n";
          $msg_size += length $msg;
        }

      } # End MSGLOOP

      close SOCK;
    } # END SOCKLOOP

    $g{'msgxfrtime'} = time - $g{'msgxfrtime'};


   # Now send our dm status message!
    if(!$g{'debug'}) { 
      my $dm_msg = dm_stat_msg();
      socket(SOCK, PF_INET, SOCK_STREAM, getprotobyname('tcp'))
        or do_log("Failed to create socket ($!)",0) and return;
      connect(SOCK, $p_addr)
        or do_log("Can't connect to display server ($!)",0) and return;
  
      print SOCK "$dm_msg\n" if !$g{'debug'};
      close SOCK;
    }
   
    do_log("Done sending messages",2);
  }


 # Spit out various data about our devmon process
  sub dm_stat_msg {
    my $this_poll_time = $g{'snmppolltime'} + $g{'testtime'} +
      $g{'msgxfrtime'};

   # Determine our number of clear msgs sent
    my $num_clears = 0;
    for my $dev (keys %{$g{'cleardata'}}) {
      $num_clears += scalar keys %{$g{'cleardata'}{$dev}};
    }
 
    my $message = "devmon, version $g{'version'}\n"        .
                  "\n"                                     .
                  "Node name:        $g{'nodename'}\n"     .
                  "Node number:      $g{'my_nodenum'}\n"   .
                  "Process ID:       $g{'mypid'}\n"        .
                  "\n"                                     .
                  "Cycle time:       $g{'cycletime'}\n"    .
                  "Dead time:        $g{'deadtime'}\n"     .
                  "\n"                                     .
                  "Polled devices:   $g{'numdevs'}\n"      .
                  "Polled tests:     $g{'numtests'}\n"     .
                  "Avg tests/node:   $g{'avgtestsnode'}\n" .
                  "# clear msgs:     $num_clears\n"        .
                  "\n"                                     .
                  "SNMP test time:   $g{'snmppolltime'}\n" .
                  "Test logic time:  $g{'testtime'}\n"     .
                  "BB msg xfer time: $g{'msgxfrtime'}\n"   .
                  "This poll period: $this_poll_time\n"    .
                  "\n"                                     .
                  "Avg poll time:    ";

   # Calculate avg poll time over the last 5 poll cycles
    my $num_polls = scalar @{$g{'avgpolltime'}};
    if($num_polls < 5) {
      $message .= "wait\n";
    }
    else {
      my $avg_time;
      for my $time (@{$g{'avgpolltime'}}) { $avg_time += $time }
      $avg_time /= $num_polls;
      $message .= "$avg_time seconds\n\n" .
        "Poll time averaged over 5 poll cycles.";

    }

   # Add the header
    my $host = $g{'nodename'};
    $host =~ s/\./,/g; # Dont forget our FQDN stuff
    $message = "status $host.dm green ".(scalar localtime)."\n\n$message\n";

    return $message;

  }

1;
